package SM.ginf;

import org.w3c.rdf.*;
import org.ginf.*;
import org.ginf.helpers.*;
import org.ginf.delivery.HTTP.*;
import org.ginf.schemas.*;
import java.io.*;

public class HelloWorldServer2 implements Layer {

  static final String HELLO_WORLD = "Hello world!";

  /** we need the factory to create models */
  RDFFactory factory;

  public HelloWorldServer2(RDFFactory factory) {

    this.factory = factory;
    factory.registry().addLoader(new HelloWorld());
  }

  public Model dispatch(Model model, Layer from) throws Exception {

    // uncomment it to see what we really got
    //    System.err.println("\nI got the following model:");
    //    factory.createRDFMS().serialize(model, new OutputStreamWriter(System.err));

    // We want to access schema information and validate the model
    SchemaModel m = factory.createSchemaModel( model );

    try {
      // optionally validate the model
      m.validate();

      // find the request
      Resource request = m.find1( null, RDF.type, CoreComm.Message ).subject();

      // we can process only GetString requests
      if(m.find( request, RDF.type, HelloWorld.GetString ).size() > 0) {
	
	Model replyMsg = factory.createModel();
	Resource reply = createReply(m, replyMsg, HelloWorld.StringToPrint);
	replyMsg.add( reply, HelloWorld.string, HELLO_WORLD );
	
	// send back the reply
	from.dispatch( replyMsg, this );

	// hang up the client connection
	Model hangup = factory.createModel();
	createReply(m, hangup, CoreState.DiscardState);
	from.dispatch(hangup, this );
      }
      
    } catch (Exception any) {
      System.err.println(toString() + " ERROR " + any.getMessage());
    }
    // done
    return null;
  }

  public Resource createReply(Model orig, Model reply, Resource replyType) {
   
    // find the request
    Resource r = orig.find1( null, RDF.type, CoreComm.Request ).subject();
    // find the HTTP connection
    Resource state = orig.find1( null, RDF.type, HTTP.Connection ).subject();

    // create the reply message: this may look strange for now,
    // you might understand it better by having look at the HTTP mapping
    // vocabulary: http://www-diglib.stanford.edu/diglib/ginf/1999/05/26-http

    Resource e = RDFUtil.noname();
    reply.add( e, RDF.type, replyType );
    reply.add( e, CoreState.state, state );
    reply.add( state, RDF.type, HTTP.Connection );
    return e;
  }

  public String toString() {
    return "Hello World Server";
  }


  public static void main(String[] args) {
    
    // create the default factory
    RDFFactory factory = new RDFFactoryImpl();
    
    // create the server
    HelloWorldServer2 s = new HelloWorldServer2(factory);
    // create the transport layer
    HTTPLayer h = new HTTPLayer( factory );
    
    // connect transport to the server
    h.connect( "dispatch", s );
    
    try {
      
      h.acceptConnections();
      
    } catch (Exception any) {
      System.err.println(s.toString() + " Could not launch: " + any);
    }
  }
}
