/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package org.ginf.delivery.HTTP;

import java.io.*;

/**
 * Decodes an escaped stream.
 *
 * @author Sergey Melnik <melnik@db.stanford.edu>
 */
public class HTTPDecoder extends FilterInputStream {

  static final char ESCAPE = '%';
  
  public HTTPDecoder(InputStream in) {
    super(in);
  }
  
  boolean done = false;

  /**
   * Read a single character.
   *
   * @exception  IOException  If an I/O error occurs
   */
  public int read() throws IOException {

    if(done)
      return -1;

    int c = in.read();

    if( c == 10 ) {
      done = true;
      return -1;
    }

    if(c == ESCAPE) {
      int c1 = toInt(in.read()); // hex -> int
      int c2 = toInt(in.read());
      //      System.err.println("C1=" + c1 + ", C2=" + c2 + ": " + ((c1 << 4) + c2));
      c = (c1 << 4) + c2;

    } else if(c == '+')
      c = ' ';

    return c;
  }

  int toInt(int hex) {

    if(hex >= (int)'0' && hex <= (int)'9')
      return hex - (int)'0';
    if(hex >= (int)'A' && hex <= (int)'F')
      return hex - (int)'A' + 10;
    return -1;
  }  
  /**
   * Read characters into a portion of an array.
   *
   * @exception  IOException  If an I/O error occurs
   */
  public int read(byte cbuf[], int off, int len) throws IOException {
    
    int i = 0;
    while (len > 0) {
      int c = read();
      if(c >= 0)
	cbuf[off + i++] = (byte)c;
      else
	return i > 0 ? i : -1;
      len--;
    }
    return i;
  }
  
  /**
   * Tell whether this stream supports the mark() operation.
   */
  public boolean markSupported() {
    return false;
  }

  /**
   * Closes this input stream and releases any system resources 
     * associated with the stream. The <code>close</code> method of 
     * <code>FilterInputStream</code> calls the <code>close</code> method 
     * of its underlying input stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterInputStream#in
     * @since      JDK1.0
     */
  public void close() throws IOException {
    // A SAX driver may attempt to close the stream
    // after parsing. We do not allow this!
    //    in.close();
  }

  
}
