/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package org.ginf.delivery.HTTP;

import java.io.*;

/**
 * Encodes a stream into an escaped representation.
 *
 * @author Sergey Melnik <melnik@db.stanford.edu>
 */
public class HTTPEncoder extends FilterOutputStream {

  static final char ESCAPE = '%';
  
  public HTTPEncoder(OutputStream in) {
    super(in);
  }

  int toHex(int n) {
    if(n <= 9)
      return n + (int)'0';
    else // A-F
      return (n-10) + (int)'A';
  }
  
  public void write(int b) throws IOException {

    if(b == ' ')
      out.write((int)'+');
    
    else if(b < (int)'0' ||
	    b > (int)'z') {

      out.write(ESCAPE);
      out.write( toHex(b >> 4) );
      out.write( toHex(b & (0xF)) );

    } else
      out.write( b );
  }

  public void write(byte b[], int off, int len) throws IOException {
    
    while (len > 0) {

      write(b[off++]);
      len--;
    }
  }
  
  
}
