/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package org.ginf.helpers;

import org.xml.sax.InputSource;
import org.w3c.rdf.*;
import org.ginf.*;

import java.util.*;
import java.net.*;
import java.io.*;

/**
 * Alias loader. Allows to fetch schemas from an alternative URI.
 * 
 * @author Sergey Melnik <melnik@db.stanford.edu>
 */

public class AliasLoader implements SchemaLoader {


  Hashtable aliases = new Hashtable();

  /**
   * Sets an alias for the schema URI.
   * Can be used to fetch schemas more efficiently from a file etc.
   */
  public void setAlias(String orig, String alias) {
    
    aliases.put(orig, alias);
  }

  // SchemaLoader implementation

  public boolean loadSchema(String uri, RDFFactory factory) {

    //    System.err.println("--- CALLED " + aliases.size());
    
    // lookup alias
    String alias = (String)aliases.get(uri);

    if(alias != null) {
      
      // prevent from fetching an unavailable schema again and again
      factory.registry().addKnown( alias );
      
      try {
	
	System.err.println("CONNECTING (alias): " + alias);
	InputSource source = new InputSource( new URL(alias).openStream() );
	source.setSystemId(uri);
	factory.registry().fetch(source);
	
      } catch (Exception any) {
	
 	System.err.println("Schema " + uri + " is unavailable: " + any);
	factory.registry().remove(alias);
      }
      return true;
      
    } else
      return false;
  }

}
